package com.jeepay.paydemo;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;
import android.widget.EditText;

import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jeequan.jeepay.JeepayClient;
import com.jeequan.jeepay.model.PayOrderCreateReqModel;
import com.jeequan.jeepay.request.PayOrderCreateRequest;
import com.jeequan.jeepay.response.PayOrderCreateResponse;
import com.tencent.mm.opensdk.constants.Build;
import com.tencent.mm.opensdk.modelbiz.WXLaunchMiniProgram;
import com.tencent.mm.opensdk.openapi.IWXAPI;
import com.tencent.mm.opensdk.openapi.WXAPIFactory;

import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.Date;


public class MainActivity extends Activity{
	
    private IWXAPI api;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
		checkPermission();
		api = WXAPIFactory.createWXAPI(this, Constants.APP_ID, false);
		api.registerApp(Constants.APP_ID);

		// 跳转微信小程序支付
		View jumpToWxLitePay = findViewById(R.id.jump_to_wx_lite_pay);
		jumpToWxLitePay.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {

				int wxSdkVersion = api.getWXAppSupportAPI();
				if (wxSdkVersion < Build.OFFLINE_PAY_SDK_INT) {
					Utils.show(getApplicationContext(), "not supported");
					return;
				}

				// 支付金额
				EditText payAmountText = findViewById(R.id.pay_amount);
				String payAmount = payAmountText.getText().toString();
				if(null == payAmount || "".equals(payAmount)) {
					Utils.show(getApplicationContext(), "支付金额不能为空");
					return;
				}

				new Thread(new Runnable() {
					@Override
					public void run() {

						String payData = pay("wxapp", payAmount);
						if(null == payData || "".equals(payData)) {
							return;
						}

						JSONObject liteObj = JSON.parseObject(payData);
						String wxLiteGhid = liteObj.getString("ghid");	// 小程序原始ID
						String wxLitePath = liteObj.getString("path"); // 小程序路径
						String wxLiteEnv = liteObj.getString("env");	// 小程序环境
						String qrUrl = liteObj.getString("qrUrl");     // 聚合码支付URL

						WXLaunchMiniProgram.Req req = new WXLaunchMiniProgram.Req();
						req.userName = wxLiteGhid; // 填小程序原始id

						req.miniprogramType = WXLaunchMiniProgram.Req.MINIPTOGRAM_TYPE_RELEASE;
						if("test".equals(wxLiteEnv)) { // 开发版
							req.miniprogramType = WXLaunchMiniProgram.Req.MINIPROGRAM_TYPE_TEST;
						}else if("preview".equals(wxLiteEnv)) { // 体验版
							req.miniprogramType = WXLaunchMiniProgram.Req.MINIPROGRAM_TYPE_PREVIEW;
						}else if("release".equals(wxLiteEnv)) { // 正式版
							req.miniprogramType = WXLaunchMiniProgram.Req.MINIPTOGRAM_TYPE_RELEASE;
						}
						//拉起小程序页面的可带参路径，不填默认拉起小程序首页，对于小游戏，可以只传入 query 部分，来实现传参效果，如：传入 "?foo=bar"。
						req.path = wxLitePath + "?q=" + qrUrl;
						// 打开小程序请求
						api.sendReq(req);

					}
				}).start();

			}
		});

		// 跳转支付宝小程序支付
		View jumpToAliLitePay = findViewById(R.id.jump_to_ali_lite_pay);
		jumpToAliLitePay.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				EditText payAmountText = findViewById(R.id.pay_amount);
				String payAmount = payAmountText.getText().toString();
				if(null == payAmount || "".equals(payAmount)) {
					Utils.show(getApplicationContext(), "支付金额不能为空");
					return;
				}

				new Thread(new Runnable() {
					@Override
					public void run() {

						String payData = pay("aliapp", payAmount);
						if(null == payData || "".equals(payData)) {
							return;
						}

						JSONObject liteObj = JSON.parseObject(payData);
						String appId = liteObj.getString("appId");		// 小程序appid
						String page = liteObj.getString("path");		// 小程序路径
						String qrUrl = liteObj.getString("qrUrl");		// 聚合码支付URL

						startAliLite(getApplicationContext(), appId, page, "qrCode=" + qrUrl);
					}
				}).start();


			}
		});

		// H5转微信小程序支付
		View jumpToWxH5Pay = findViewById(R.id.jump_to_wx_h5_pay);
		jumpToWxH5Pay.setOnClickListener(new View.OnClickListener() {

			@Override
			public void onClick(View v) {
				EditText payAmountText = findViewById(R.id.pay_amount);
				String payAmount = payAmountText.getText().toString();
				if(null == payAmount || "".equals(payAmount)) {
					Utils.show(getApplicationContext(), "支付金额不能为空");
					return;
				}

				new Thread(new Runnable() {
					@Override
					public void run() {

						String payData = pay("wxh5", payAmount);
						if(null == payData || "".equals(payData)) {
							return;
						}
						Uri uri = Uri.parse(payData);
						Intent intent = new Intent(Intent.ACTION_VIEW, uri);
						intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK );
						startActivity(intent);
					}
				}).start();


			}
		});

	}

	/**
	 * 支付宝小程序
	 *
	 * @param context 上下文处理
	 * @param appId   小程序appId
	 * @param page    小程序跳转的页面。如果不设置，默认为跳转至首页。例如：pages/user/user
	 * @param query   启动参数，内容按照格式为参数名=参数值&参数名=参数值…之后encode，例如：client_type=2
	 */
	public static void startAliLite(Context context, String appId, String page, String query) {
		try {
			StringBuffer sb = new StringBuffer("alipays://platformapi/startapp?");
			sb.append("appId=").append(appId).append("&");
			sb.append("page=").append(page).append("&");
			String link = URLEncoder.encode(query, "UTF-8");
			sb.append("query=").append(link);
			Uri uri = Uri.parse(sb.toString());
			Intent intent = new Intent(Intent.ACTION_VIEW, uri);
			intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK );
			context.startActivity(intent);
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		}
	}

	/**
     * 调用jeepay统一下单
	 *
	 * @param entryLiteType 打开小程序类型
	 * @param payAmount   支付金额
	 * @return
     */
	public String pay(String entryLiteType, String payAmount) {
		JeepayClient jeepayClient = JeepayClient.getInstance(Constants.appId, Constants.apiKey, Constants.basePayUrl);
		String wayCode = "QR_CASHIER";                      // 支付方式
		PayOrderCreateRequest request = new PayOrderCreateRequest();
		PayOrderCreateReqModel model = new PayOrderCreateReqModel();
		model.setMchNo(Constants.mchNo);                    // 商户号
		model.setAppId(jeepayClient.getAppId());            // 应用ID
		String orderNo = "AND" + new Date().getTime();
		model.setMchOrderNo(orderNo);                       // 商户订单号
		model.setWayCode(wayCode);                          // 支付方式
		model.setAmount(new BigDecimal(payAmount).multiply(new BigDecimal(100)).longValue());        // 金额，单位分
		model.setCurrency("CNY");                           // 币种，目前只支持cny
		model.setClientIp("192.166.1.132");                 // 发起支付请求客户端的IP地址
		model.setSubject("商品标题");                        // 商品标题
		model.setBody("商品描述");                           // 商品描述

		JSONObject obj = new JSONObject();
		obj.put("entryLiteType", entryLiteType);
		model.setChannelExtra(obj.toJSONString());       	// 渠道扩展参数
		model.setExtParam("");                              // 商户扩展参数,回调时原样返回

		String payDate = "";
		request.setBizModel(model);
		try {
			PayOrderCreateResponse response = jeepayClient.execute(request);
			// 下单成功
			if(response.isSuccess(Constants.apiKey)) {
				payDate = response.get().getPayData();
			}else {
				Utils.show(getApplicationContext(), "下单失败:" + response.get().getErrMsg());
			}
		} catch (Exception e) {
			Utils.show(getApplicationContext(), "下单失败:" + e.getMessage());
		}

		return payDate;
	}


	private void checkPermission() {
		int permissionCheck = ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE);
		if (permissionCheck != PackageManager.PERMISSION_GRANTED) {
			ActivityCompat.requestPermissions(this,
					new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE},
					Constants.PERMISSIONS_REQUEST_STORAGE);
		}
	}

	@Override
	public void onRequestPermissionsResult(int requestCode,
										   String permissions[], int[] grantResults) {
		switch (requestCode) {
			case Constants.PERMISSIONS_REQUEST_STORAGE: {
				if (grantResults.length > 0
						&& grantResults[0] == PackageManager.PERMISSION_GRANTED) {
				} else {
					Utils.show(getApplicationContext(), "Please give me storage permission!");
				}
				return;
			}
		}
	}

}